#include "armada.h"

const int PROBA_TIR=5; // Inverse de la probabilite de tirer si possible

// Ajoute un vaisseau a l'armada
void ajoute(Armada& a, Vaisseau v) {
    assert(a.n < MAX_VAISSEAUX);
    a.v[a.n++] = v;
}

// Affichage de l'armada (vaisseaux et tirs)
void affiche(Armada a) {
    for(int i=0; i<a.n; i++)
        affiche(a.v[i]);
    for(int i=0; i<NB_TIRS; i++)
        affiche(a.tirs[i]);
}

// Effacage de l'armada
void efface(Armada a) {
    for(int i=0; i<a.n; i++)
        efface(a.v[i]);
    for(int i=0; i<NB_TIRS; i++)
        efface(a.tirs[i]);
}

// Extremites gauche, droite et bas occupees par les vaisseaux de l'armada
void bornes(Armada a, int& gauche, int& droite, int& bas) {
    gauche = largeur; droite=-1; bas=-1;
    for(int i=0; i<a.n; i++) {
        if(a.v[i].vivant && gauche > a.v[i].x)
            gauche = a.v[i].x;
        if(a.v[i].vivant && droite < a.v[i].x)
            droite = a.v[i].x;
        if(a.v[i].vivant && bas < a.v[i].y)
            bas = a.v[i].y;
    }
}

// Feu ! Le tir sera gere par a.tirs[iTir]
void nouveau_tir(Armada& a, int iTir) {
    // Trouve les vaisseaux au plus bas dans chaque colonne de cases
    int index[largeur];
    for(int i=0; i<largeur; i++)
        index[i]=-1;
    for(int i=0; i<a.n; i++)
        if(a.v[i].vivant &&
           (index[a.v[i].x]<0 || a.v[i].y > a.v[index[a.v[i].x]].y))
            index[a.v[i].x] = i;

    // Compte-les
    int n=0;
    for(int i=0; i<largeur; i++)
        if(index[i]>=0)
            ++n;

    // Choisis un de ces vaisseaux au hasard pour tirer
    int tireur = rand() % n;
    int iTireur=0;
    n=0;
    for(int i=0; n<=tireur; i++) {
        assert(i<largeur);
        if(index[i]>=0) {
            iTireur=index[i];
            ++n;
        }
    }

    // Feu !
    a.tirs[iTir].x = a.v[iTireur].x;
    a.tirs[iTir].y = a.v[iTireur].y;
    a.tirs[iTir].dy = +1;
}

// Deplacement de l'armada et de ses tirs.
// Indique si l'armada n'a pas encore atteint son but, le bas de l'ecran.
bool deplace(Armada& a) {
    int g, d, b;
    bornes(a, g, d, b);
    int dx=0, dy=0;
    if((a.droite && d==largeur-1) || (!a.droite && g==0)) {
        ++dy;
        a.droite = !a.droite;
    } else if(a.droite)
        ++dx;
    else
        --dx;
    for(int i=0; i<a.n; i++)
        deplace(a.v[i],dx,dy);

    for(int i=0; i<NB_TIRS; i++)
        if(actif(a.tirs[i]))
            deplace(a.tirs[i]);
        else if(rand()%PROBA_TIR==0)
            nouveau_tir(a, i);
    return (b+dy!=hauteur-1);
}

// Gestion d'eventuel impact d'un tir sur l'armada
bool explosion(Armada& a, Tir& t) {
    for(int i=0; i<a.n; i++)
        if(explosion(a.v[i], t))
            return true;
    return false;
}
