// Imagine++ project
// Project:  Space Invaders
// Author:   Pascal Monasse
// Date:     2018/11/09

#include "armada.h"
#include <iostream>
#include <cstdlib>
using namespace std;

const Color colors[] = {RED, GREEN, BLUE, MAGENTA, CYAN};

// Cree des tirs inactifs pour l'aramda
void init_tirs(Armada& a) {
    for(int i=0; i<NB_TIRS; i++) {
        a.tirs[i].x=-1;
        a.tirs[i].y=-1;
        a.tirs[i].dy=0;
    }
}

// Cree une armada
void forme_armada(Armada& a) {
    for(int i=0; i<5; i++)
        for(int j=0; j<11; j++) {
            Vaisseau v = {largeur/2+2*j-11,i,colors[rand()%5],true};
            ajoute(a, v);
        }
    init_tirs(a);
}

// Cree un bunker centre en abscisse x
void forme_bunker(Armada& bunker, int x) {
    bunker.n=0;
    for(int i=x-1; i<=x+1; i++) {
        Vaisseau v = {i, hauteur-5, YELLOW, true};
        ajoute(bunker,v);
    }
    for(int j=0; j<2; j++)
        for(int i=x-2; i<=x+2; i++) {
            Vaisseau v = {i, hauteur-4+j, YELLOW, true};
            ajoute(bunker,v);
    }
    for(int i=x-2; i<=x+2; i+=4) {
        Vaisseau v = {i, hauteur-2, YELLOW, true};
        ajoute(bunker,v);
    }
    init_tirs(bunker);
}

// Gestion des evenements clavier
void Clavier(int& dx, bool& tir) {
    dx = 0; tir=false;
    Event e;
    do {
        getEvent(0, e);
        if(e.type!=EVT_KEY_ON) continue;
        if(e.key=='s' || e.key=='S') dx = -1;
        if(e.key=='d' || e.key=='D') dx = +1;
        if(e.key==' ') tir=true;
    } while(e.type!=0 && dx==0 && !tir);
}

// Gere l'explosion eventuelle d'un vaisseau de l'aramda par le tir
void gere_explosion(Armada& a, Tir& t) {
    if(explosion(a,t)) {
        efface(t);
        efface(a);
        affiche(a);
    }
}

// Gestion des actions du capitaine
void gere_capitaine(Vaisseau& capitaine, Tir& tir) {
    if(! capitaine.vivant) return;
    int dx; bool tire;
    Clavier(dx, tire);
    if(dx!=0 & capitaine.x+dx>=0 && capitaine.x+dx<largeur) {
        efface(capitaine);
        deplace(capitaine, dx, 0);
        affiche(capitaine);
    }
    if(tire && !actif(tir)) {
        tir.x=capitaine.x;
        tir.y=hauteur-1;
        tir.dy=-1;
    }            
}

// Verifie que le capitaine n'est pas touche par un tir de l'armada
void verifie_sante_capitaine(Vaisseau& capitaine, Armada a) {
    for(int i=0; i<NB_TIRS; i++)
        explosion(capitaine, a.tirs[i]);
}

int main() {
    cout << "Touches s et d pour le mouvement, espace pour tirer" << endl;
    openWindow(largeur*zoom, hauteur*zoom, "Space Invaders");

    Armada a; a.n=0; a.droite=true;
    forme_armada(a);
    affiche(a);

    Vaisseau capitaine = {largeur/2, hauteur-1, BLACK, true};
    affiche(capitaine);

    Armada bunker[3];
    for(int i=0; i< 3; i++) {
        forme_bunker(bunker[i], (i+1)*largeur/4);
        affiche(bunker[i]);
    }

    Tir tir = {largeur/2, hauteur, 0};
    while(capitaine.vivant && a.n>0) {
        // Gestion du tir du capitaine
        efface(tir);
        deplace(tir);
        affiche(tir);
        for(int i=0; i<3; i++)
            gere_explosion(bunker[i], tir);
        gere_explosion(a,tir);

        // Action de l'armada d'envahisseurs
        efface(a);
        capitaine.vivant = deplace(a);
        affiche(a);
        for(int i=0; i<3; i++)
            for(int j=0; j<NB_TIRS; j++)
                gere_explosion(bunker[i], a.tirs[j]);
        gere_explosion(a,tir);
        verifie_sante_capitaine(capitaine, a);

        // Actions du capitaine
        gere_capitaine(capitaine, tir);
        verifie_sante_capitaine(capitaine, a);
        gere_capitaine(capitaine, tir);
        verifie_sante_capitaine(capitaine, a);
        milliSleep(100);
    }

    // Fin de partie
    if(capitaine.vivant)
        drawString(largeur*zoom/3, hauteur*zoom/2, "Gagne!", RED);
    else
        drawString(largeur*zoom/3, hauteur*zoom/2, "Perdu!", RED);

    endGraphics();
    return 0;
}
